# Kasumi: Dictionary management tool for Anthy
# Copyright (C) 2004 Takashi Nakamoto
#
# This file is part of Kasumi
#
# Kasumi is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# Please send bug-reports, opinions and patches, if any, to me by E-Mail.
# My E-mail address is following: bluedwarf@openoffice.org

require 'gtk2'

require 'wordtable.rb'
require 'io.rb'

$entry_word
$entry_yomi
$spin_freq
$combo_part
$wordtable
$noun_option_pane
$adv_option_pane
$noun_na_check
$noun_sa_check
$noun_suru_check
$noun_gokan_check
$noun_kakujoshi_check
$adv_option_pane
$adv_to_check
$adv_taru_check
$adv_suru_check
$adv_gokan_check

def delete_event(widget)
  if $modified == true then
    puts "not saved"
  end

  return false
end

def destroy(widget)
  Gtk.main_quit
end

def quit(widget)
  Gtk.main_quit unless delete_event(widget)
end

def save(widget)
  dic_str = String.new
  
  iter = $store.iter_first
  begin
    dic_str += (iter[1].strip + " " + iter[2].to_s + " " + iter[0].strip + "\n")
    dic_str += ("品詞 = " + iter[3].strip + "\n")
    iter[4].each do |key,val|
      dic_str += (key.strip + " = " + val.strip + "\n")
    end
    dic_str += "\n"
  end while iter.next!

  data = Iconv.iconv("EUC-JP","UTF-8",dic_str)
  
  write_to($dic_file,data)
  system("cat #{$dic_file} | anthy-dic-tool --load")

  $modified = false
end

def add(widget)
  iter = $store.append
  $store.set_value(iter,COL_WORD,"単語")
  $store.set_value(iter,COL_YOMI,"読み")
  $store.set_value(iter,COL_FREQ,1)
  $store.set_value(iter,COL_PART,"名詞")
  etc = {"な接続" => "n", "さ接続" => "n", "する接続" => "n", "語幹のみで文節" => "n", "格助詞接続" => "n"}
  $store.set_value(iter,COL_ETC,etc)

  $wordtable.selection.select_iter(iter)
  wordtable_pressed($wordtable)

  $modified = true
end

def remove(widget)
  if iter = $wordtable.selection.selected then
    $store.remove(iter)
  end

  $wordtable.selection.unselect_all
  wordtable_pressed($wordtable)

  $modified = true
end

def wordtable_pressed(view)
  if iter = view.selection.selected then
    $entry_word.set_text(iter[0])
    $entry_yomi.set_text(iter[1])
    $spin_freq.set_value(iter[2])
    $combo_part.entry.set_text(iter[3])

    if iter[3] == "名詞" then
      $adv_option_pane.hide
      $noun_option_pane.show
      $noun_na_check.active = ((iter[4])["な接続"] == "y")
      $noun_sa_check.active = ((iter[4])["さ接続"] == "y")
      $noun_suru_check.active = ((iter[4])["する接続"] == "y")
      $noun_gokan_check.active = ((iter[4])["語幹のみで文節"] == "y")
      $noun_kakujoshi_check.active = ((iter[4])["格助詞接続"] == "y")
    elsif iter[3] == "副詞" then
      $noun_option_pane.hide
      $adv_option_pane.show
      $adv_to_check.active = ((iter[4])["と接続"] == "y")
      $adv_taru_check.active = ((iter[4])["たる接続"] == "y")
      $adv_suru_check.active = ((iter[4])["する接続"] == "y")
      $adv_gokan_check.active = ((iter[4])["語幹のみで文節"] == "y")
    else
      $noun_option_pane.hide
      $adv_option_pane.hide
    end
  else
    $entry_word.set_text("")
    $entry_yomi.set_text("")
    $spin_freq.set_value(1)
    $combo_part.entry.set_text("")
    $noun_option_pane.hide
    $adv_option_pane.hide
  end
end

def changed_word_entry(entry)
  if iter = $wordtable.selection.selected then
    iter[0] = entry.text
  end

  $modified = true
end

def changed_yomi_entry(entry)
  if iter = $wordtable.selection.selected then
    iter[1] = entry.text
  end

  $modified = true
end

def changed_freq_spin(spin)
  if iter = $wordtable.selection.selected then
    iter[2] = spin.value
  end
  
  $modified = true  
end

def changed_part_entry(entry)
  if iter = $wordtable.selection.selected then
    iter[3] = entry.text

    if iter[3] == "名詞" then
      etc = {"な接続" => "n", "さ接続" => "n", "する接続" => "n",
        "語幹のみで文節" => "n", "格助詞接続" => "n"}
      iter[4].each_key do |key|
        etc[key] = (iter[4])[key] if etc.key?(key)
      end
      iter[4].replace(etc)
      
      $adv_option_pane.hide
      $noun_option_pane.show
      $noun_na_check.active = ((iter[4])["な接続"] == "y")
      $noun_sa_check.active = ((iter[4])["さ接続"] == "y")
      $noun_suru_check.active = ((iter[4])["する接続"] == "y")
      $noun_gokan_check.active = ((iter[4])["語幹のみで文節"] == "y")
      $noun_kakujoshi_check.active = ((iter[4])["格助詞接続"] == "y")
    elsif iter[3] == "副詞" then
      etc = {"と接続" => "n", "たる接続" => "n", "する接続" => "n",
        "語幹のみで文節" => "n"}
      iter[4].each_key do |key|
        etc[key] = (iter[4])[key] if etc.key?(key)
      end
      iter[4].replace(etc)
      
      $noun_option_pane.hide
      $adv_option_pane.show
      $adv_to_check.active = ((iter[4])["と接続"] == "y")
      $adv_taru_check.active = ((iter[4])["たる接続"] == "y")
      $adv_suru_check.active = ((iter[4])["する接続"] == "y")
      $adv_gokan_check.active = ((iter[4])["語幹のみで文節"] == "y")
    elsif iter[3] == "人名" then
      iter[4] = Hash.new
      
      $noun_option_pane.hide
      $adv_option_pane.hide
    elsif iter[3] == "地名" then
      iter[4] = Hash.new
      
      $noun_option_pane.hide
      $adv_option_pane.hide
    elsif iter[3] == "形容詞" then
      iter[4] = Hash.new
      
      $noun_option_pane.hide
      $adv_option_pane.hide
    end
  end

  $modified = true    
end

def changed_checkbutton(widget)
  iter = $wordtable.selection.selected
  (iter[4])[widget.label] = widget.active? ? "y" : "n"
end

def create_top_window
  window = Gtk::Window.new(Gtk::Window::TOPLEVEL)
  window.set_title("Kasumi")
  window.signal_connect('delete_event') do |widget,data|
    delete_event(widget)
  end
  window.signal_connect('destroy') do |widget|
    destroy(widget)
  end

  vbox = Gtk::VBox.new(false,8)
  vbox.set_border_width(8)
  window.add(vbox)
  vbox.show

  hbox = Gtk::HBox.new(false,8)
  vbox.pack_start(hbox, true, true, 0)
  hbox.show

  scrolledwindow = Gtk::ScrolledWindow.new(nil,nil)
  scrolledwindow.set_policy(Gtk::POLICY_AUTOMATIC,Gtk::POLICY_AUTOMATIC)
  scrolledwindow.set_size_request(300,300)
  hbox.pack_start(scrolledwindow,true,true,0)
  scrolledwindow.show

  $wordtable = create_wordtable
  scrolledwindow.add($wordtable)
  $wordtable.signal_connect('cursor-changed') do |view,data|
    wordtable_pressed(view)
  end
  $wordtable.show
  
  sub_vbox = Gtk::VBox.new(false,0)
  hbox.pack_start(sub_vbox, false, false, 0)
  sub_vbox.show

  label = Gtk::Label.new("単語")
  alignment = Gtk::Alignment.new(0,0.5,0,0)
  alignment.add(label)
  sub_vbox.pack_start(alignment, false, false, 0)
  alignment.show
  label.show

  $entry_word = Gtk::Entry.new
  sub_vbox.pack_start($entry_word, false, false, 0)
  $entry_word.signal_connect('changed') do |widget|
    changed_word_entry(widget)
  end
  $entry_word.show

  label = Gtk::Label.new("読み")
  alignment = Gtk::Alignment.new(0,0.5,0,0)
  alignment.add(label)
  sub_vbox.pack_start(alignment, false, false, 0)
  alignment.show
  label.show

  $entry_yomi = Gtk::Entry.new
  sub_vbox.pack_start($entry_yomi, false, false, 0)
  $entry_yomi.signal_connect('changed') do |widget|
    changed_yomi_entry(widget)
  end
  $entry_yomi.show

  label = Gtk::Label.new("頻度")
  alignment = Gtk::Alignment.new(0,0.5,0,0)
  alignment.add(label)
  sub_vbox.pack_start(alignment, false, false, 0)
  alignment.show
  label.show

  adjustment = Gtk::Adjustment.new(1,1,32767,1,100,0)
  $spin_freq = Gtk::SpinButton.new(adjustment, 1.0, 0)
  sub_vbox.pack_start($spin_freq, false, false, 0)
  $spin_freq.signal_connect('value-changed') do |widget|
    changed_freq_spin(widget)
  end
  $spin_freq.show
  
  label = Gtk::Label.new("品詞")
  alignment = Gtk::Alignment.new(0,0.5,0,0)
  alignment.add(label)
  sub_vbox.pack_start(alignment, false, false, 0)
  alignment.show
  label.show

  $combo_part = Gtk::Combo.new(["名詞","副詞","人名","地名","形容詞"])
  sub_vbox.pack_start($combo_part, false, false, 0)
  $combo_part.entry.set_editable(false)
  $combo_part.entry.signal_connect('changed') do |widget|
    changed_part_entry(widget)
  end
  $combo_part.show

  $noun_option_pane = Gtk::VBox.new(false,0)
  sub_vbox.pack_start($noun_option_pane,false,false,0)

  $noun_na_check = Gtk::CheckButton.new("な接続")
  $noun_option_pane.pack_start($noun_na_check,false,false,0)
  $noun_na_check.signal_connect('toggled') do |widget|
    changed_checkbutton(widget)
  end
  $noun_na_check.show

  $noun_sa_check = Gtk::CheckButton.new("さ接続")
  $noun_option_pane.pack_start($noun_sa_check,false,false,0)
  $noun_sa_check.signal_connect('toggled') do |widget|
    changed_checkbutton(widget)
  end
  $noun_sa_check.show

  $noun_suru_check = Gtk::CheckButton.new("する接続")
  $noun_option_pane.pack_start($noun_suru_check,false,false,0)
  $noun_suru_check.signal_connect('toggled') do |widget|
    changed_checkbutton(widget)
  end
  $noun_suru_check.show

  $noun_gokan_check = Gtk::CheckButton.new("語幹のみで文節")
  $noun_option_pane.pack_start($noun_gokan_check,false,false,0)
  $noun_gokan_check.signal_connect('toggled') do |widget|
    changed_checkbutton(widget)
  end
  $noun_gokan_check.show

  $noun_kakujoshi_check = Gtk::CheckButton.new("格助詞接続")
  $noun_option_pane.pack_start($noun_kakujoshi_check,false,false,0)
  $noun_kakujoshi_check.signal_connect('toggled') do |widget|
    changed_checkbutton(widget)
  end
  $noun_kakujoshi_check.show

  $adv_option_pane = Gtk::VBox.new(false,0)
  sub_vbox.pack_start($adv_option_pane,false,false,0)

  $adv_to_check = Gtk::CheckButton.new("と接続")
  $adv_option_pane.pack_start($adv_to_check,false,false,0)
  $adv_to_check.signal_connect('toggled') do |widget|
    changed_checkbutton(widget)
  end
  $adv_to_check.show

  $adv_taru_check = Gtk::CheckButton.new("たる接続")
  $adv_option_pane.pack_start($adv_taru_check,false,false,0)
  $adv_taru_check.signal_connect('toggled') do |widget|
    changed_checkbutton(widget)
  end
  $adv_taru_check.show

  $adv_suru_check = Gtk::CheckButton.new("する接続")
  $adv_option_pane.pack_start($adv_suru_check,false,false,0)
  $adv_suru_check.signal_connect('toggled') do |widget|
    changed_checkbutton(widget)
  end
  $adv_suru_check.show

  $adv_gokan_check = Gtk::CheckButton.new("語幹のみで文節")
  $adv_option_pane.pack_start($adv_gokan_check,false,false,0)
  $adv_gokan_check.signal_connect('toggled') do |widget|
    changed_checkbutton(widget)
  end
  $adv_gokan_check.show
  
  hbutton_box = Gtk::HButtonBox.new
  hbutton_box.set_layout_style(Gtk::ButtonBox::SPREAD)
  vbox.pack_start(hbutton_box, false, false, 0)
  hbutton_box.show

  button = Gtk::Button.new("終了")
  hbutton_box.pack_start(button, false, false, 0)
  button.signal_connect('clicked') do |widget|
    quit(widget)
  end
  button.show

  button = Gtk::Button.new("保存")
  hbutton_box.pack_start(button, false, false, 0)
  button.signal_connect('clicked') do |widget|
    save(widget)
  end
  button.show

  button = Gtk::Button.new("追加")
  hbutton_box.pack_start(button, false, false, 0)
  button.signal_connect('clicked') do |widget|
    add(widget)
  end
  button.show
  
  button = Gtk::Button.new("削除")
  hbutton_box.pack_start(button, false, false, 0)
  button.signal_connect('clicked') do |widget|
    remove(widget)
  end
  button.show

  window.show
end
